library(ggplot2)
library(ncdf4)
library(trend)
library(loadeR)
library(transformeR)
library(visualizeR)
library(RColorBrewer)

rm(list=ls()) ; gc()

#Working directory
dir <- "/esarchive/scratch/molmo/CLIMCAT_MJ/CMIP6/"
OUTPUTS <- "/esarchive/scratch/molmo/CLIMCAT_MJ/OUTPUTS2/"
setwd(dir)

invariable <- loadGridData(dataset = "/esarchive/scratch/molmo/mask_1_Med_bil.nc",
                           var="precip") %>% binaryGrid(condition = "GE",threshold=0)

#-------Annual indices---------#
#Useful information
indices <- c("wsdi","csdi","tr",
             "r20mm","prcptot","cdd","cwd")
files <- intersect(list.files(pattern="2100.nc",recursive = T),list.files(pattern="yr",recursive = T))
files <- intersect(files,list.files(pattern="ssp585",recursive=T))
files2 <- intersect(list.files(pattern="ERA5"),list.files(pattern="yr"))
files <- c(files,files2)

ssn <- "Annual"
seasons <- list(c(1:12))
Model <- NA

trend.indices <- list()
series.indices <- list()

for(i in 1:length(indices)){
  print(indices[i])
  files_subset <- intersect(files,list.files(pattern=indices[i],recursive=T))
  trend.indices[[i]] <- list()
  series.indices[[i]] <- list()
  
  for(m in 1:length(files_subset)){
    print(files_subset[m])
    
    trend.indices[[i]][[m]] <- list()
    series.indices[[i]][[m]] <- list()
    
    for(s in 1:length(seasons)){
      di <- dataInventory(files_subset[m])
      
      ini <- nchar(names(di))                            
      out <- nchar("_historical-ssp585_r1i1p1f1_1950-2100.nc") 
      Model[m] <- substr(files_subset[m],start=ini+32,stop=nchar(files_subset[m])-out)
      
      data <- loadGridData(files_subset[m],var=names(di),years=1950:2014)
      
      #Interpolate into a common grid and mask ocean
      data <- interpGrid(data,new.coordinates = getGrid(invariable),method = "nearest")
     # for(y in 1:dim(data$Data)[1]){
      #  data$Data[y,,] <- data$Data[y,,]*invariable$Data[1,,]
      #}
      
      trends <- matrix(NA,length(data$xyCoords$y),length(data$xyCoords$x))
      tau <- matrix(NA,length(data$xyCoords$y),length(data$xyCoords$x))
      signif <- matrix(NA,dim(trends)[1],dim(trends)[2])
      
      for(j in 1:dim(trends)[1]){
        for(k in 1:dim(trends)[2]){
          if(!is.na(mean(data$Data[,j,k],na.rm=T))){
            trends[j,k] <- lm(data$Data[,j,k]~c(1:length(unique(getYearsAsINDEX(data)))))[["coefficients"]][2]
            tau[j,k] <- mk.test(na.omit(data$Data[,j,k]),continuity = T)[["estimates"]][["tau"]]
            signif[j,k] <- mk.test(na.omit(data$Data[,j,k]),continuity = T)$p.value
          }
        }
      }
      
      datita <- data.frame(Trend=as.numeric(trends)*10/length(unique(getYearsAsINDEX(data))),
                           Tau=as.numeric(tau),
                           Significance=as.numeric(signif),
                           Season=rep(ssn[s],length(as.numeric(trends))),
                           Model=rep(Model[m],length(as.numeric(trends))),
                           Longitude=rep(data$xyCoords$x,each=length(data$xyCoords$y)),
                           Latitude=rep(data$xyCoords$y,length(data$xyCoords$x)),
                           Index=rep(indices[i],length(as.numeric(trends))))
      
      #Serie regional
      EuroMediterranean <- aggregateGrid(data,aggr.spatial = list(FUN="mean",na.rm=T))
      Catalonia <- aggregateGrid(subsetGrid(data,latLim = c(40,43),lonLim = c(0,4)),aggr.spatial = list(FUN="mean",na.rm=T))
      
      Serie <- data.frame(Serie=c(EuroMediterranean$Data,Catalonia$Data),
                          Season=ssn[s],
                          Index=indices[i],
                          Model=Model[m],
                          Region=rep(c("EuroMediterranean","Catalonia"),each=length(getYearsAsINDEX(Catalonia))),
                          Year=rep(getYearsAsINDEX(Catalonia),2),
                          Tau=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)[["estimates"]][["tau"]],
                                    mk.test(na.omit(Catalonia$Data),continuity = T)[["estimates"]][["tau"]]),each=length(getYearsAsINDEX(Catalonia))),
                          pv=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)$p.value,
                                   mk.test(na.omit(Catalonia$Data),continuity = T)$p.value),each=length(getYearsAsINDEX(Catalonia))))
      
      trend.indices[[i]][[m]][[s]] <- datita #tal vez acá va el data.frame
      series.indices[[i]][[m]][[s]] <- Serie
      gc()
      
      if(indices[i]%in%c("cdd")){paleta=rev(brewer.pal(11,"BrBG")); range=c(-0.2,0.2)}
      if(indices[i]%in%c("cwd","r20mm")){paleta=c(brewer.pal(11,"BrBG")); range=c(-0.2,0.2)}
      if(indices[i]%in%c("prcptot")){paleta=brewer.pal(11,"BrBG"); range=c(-3,3)}
      if(indices[i]%in%c("wsdi","tr")){paleta=rev(brewer.pal(11,"RdYlBu")); range=c(-0.8,0.8)}
      if(indices[i]%in%c("csdi")){paleta=brewer.pal(11,"RdYlBu"); range=c(-0.8,0.8)}
      
      # png(filename = paste(OUTPUTS,"Trend_1950-2014_",ssn[s],"_",indices[i],"_",Model[m],".png",sep=""),res=600,width=3000,height=2000)
      # 
      # if(dim(subset(datita,Significance<0.05))[1]!=0){
      #   print(ggplot(data=datita,aes(x=Longitude,y=Latitude,fill=Trend)) + geom_contour_fill(bins=20)  +
      #           scale_fill_gradientn(name=paste(di[[names(di)]][["Units"]],"decade",sep="/"),colours=paleta,limits=range,na.value = "white")+
      #           coord_quickmap(xlim=c(min(datita$Longitude),max(datita$Longitude)),
      #                          ylim=c(min(datita$Latitude),max(datita$Latitude)),expand=F)+ borders() +
      #           theme_bw() + ggtitle(paste(ssn[s],indices[i],Model[m],sep=" "))+geom_text(data=subset(datita,Significance<=0.05),aes(label="*",fontface="bold")))
      #   dev.off()}else{
      #     print(ggplot(data=datita,aes(x=Longitude,y=Latitude,fill=Trend)) + geom_contour_fill(bins=20)  +
      #             scale_fill_gradientn(name=paste(di[[names(di)]][["Units"]],"decade",sep="/"),colours=paleta,limits=range,na.value = "white")+
      #             coord_quickmap(xlim=c(min(datita$Longitude),max(datita$Longitude)),
      #                            ylim=c(min(datita$Latitude),max(datita$Latitude)),expand=F)+ borders() + theme_bw() + ggtitle(paste(ssn[s],indices[i],Model[m],sep=" ")))#+geom_text(data=subset(datita,Significance<=0.05),aes(label="*",fontface="bold")))
      #  }
    }
  }
}
save(trend.indices,file="trend.indices.yr.CMIP6.map.historical.Rdata")
save(series.indices,file="series.indices.yr.CMIP6.map.historical.Rdata")

#--------Seasonal indices--------------------#
files <- intersect(list.files(pattern="210012.nc",recursive = T),list.files(pattern="",recursive = T))
files <- intersect(files,list.files(pattern="ssp585",recursive=T))
files2 <- intersect(list.files(pattern="ERA5"),list.files(pattern="mon"))
files <- c(files,files2)

seasons <- list(c(12,1,2),c(3,4,5),c(6,7,8),c(9,10,11)) #DJF #MAM #JJA #SON #ANNUAL
ssn <- c("Winter","Spring","Summer","Autumn")#,"Annual")
indices <- c("txx","txn","tnx","tnn","tx90p","tx10p","tn90p","tn10p",
             "rx5day","rx1day")
Model <- NA

trend.indices <- list()
series.indices <- list()

for(i in 1:length(indices)){
  print(indices[i])
  files_subset <- intersect(files,list.files(pattern=indices[i],recursive = T))
  trend.indices[[i]] <- list()
  series.indices[[i]] <- list()
  
  for(m in 1:length(files_subset)){
    print(files_subset[m])
    
    trend.indices[[i]][[m]] <- list()
    series.indices[[i]][[m]] <- list()
    
    for(s in 1:length(seasons)){
      di <- dataInventory(files_subset[m])
      
      ini <- nchar(names(di))                            
      out <- nchar("_historical-ssp585_r1i1p1f1_195001-210012.nc") 
      Model[m] <- substr(files_subset[m],start=ini+33,stop=nchar(files_subset[m])-out)
      
      data <- loadGridData(files_subset[m],var=names(di),years=1950:2014)
      
      #Interpolate into a common grid and mask ocean
      data <- interpGrid(data,new.coordinates = getGrid(invariable),method = "nearest")
      # for(y in 1:dim(data$Data)[1]){
      #   data$Data[y,,] <- data$Data[y,,]*invariable$Data[1,,]
      # }
      
      data <- aggregateGrid(data,aggr.s = list(FUN=list("mean",na.rm=T),season=seasons[[s]]))
      
      trends <- matrix(NA,length(data$xyCoords$y),length(data$xyCoords$x))
      tau <- matrix(NA,length(data$xyCoords$y),length(data$xyCoords$x))
      signif <- matrix(NA,dim(trends)[1],dim(trends)[2])
      
      for(j in 1:dim(trends)[1]){
        for(k in 1:dim(trends)[2]){
          if(!is.na(mean(data$Data[,j,k],na.rm=T))){
            trends[j,k] <- lm(data$Data[,j,k]~c(1:length(unique(getYearsAsINDEX(data)))))[["coefficients"]][2]
            tau[j,k] <- mk.test(data$Data[,j,k],continuity = T)[["estimates"]][["tau"]]
            signif[j,k] <- mk.test(data$Data[,j,k],continuity = T)$p.value
          }
        }
      }
      
      datita <- data.frame(Trend=as.numeric(trends)*10/length(unique(getYearsAsINDEX(data))),
                           Tau=as.numeric(tau),
                           Significance=as.numeric(signif),
                           Season=rep(ssn[s],length(as.numeric(trends))),
                           Model=rep(Model[m],length(as.numeric(trends))),
                           Longitude=rep(data$xyCoords$x,each=length(data$xyCoords$y)),
                           Latitude=rep(data$xyCoords$y,length(data$xyCoords$x)),
                           Index=rep(indices[i],length(as.numeric(trends))))
      
      #Serie regional
      EuroMediterranean <- aggregateGrid(data,aggr.spatial = list(FUN="mean",na.rm=T))
      Catalonia <- aggregateGrid(subsetGrid(data,latLim = c(40,43),lonLim = c(0,4)),aggr.spatial = list(FUN="mean",na.rm=T))
      
      Serie <- data.frame(Serie=c(EuroMediterranean$Data,Catalonia$Data),
                          Season=ssn[s],
                          Index=indices[i],
                          Model=Model[m],
                          Region=rep(c("EuroMediterranean","Catalonia"),each=length(getYearsAsINDEX(Catalonia))),
                          Year=rep(getYearsAsINDEX(Catalonia),2),
                          Tau=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)[["estimates"]][["tau"]],
                                    mk.test(na.omit(Catalonia$Data),continuity = T)[["estimates"]][["tau"]]),each=length(getYearsAsINDEX(Catalonia))),
                          pv=rep(c(mk.test(na.omit(EuroMediterranean$Data),continuity = T)$p.value,
                                   mk.test(na.omit(Catalonia$Data),continuity = T)$p.value),each=length(getYearsAsINDEX(Catalonia))))
      
      trend.indices[[i]][[m]][[s]] <- datita #tal vez acá va el data.frame
      series.indices[[i]][[m]][[s]] <- Serie
      gc()
      
      if(di[[names(di)]][["Units"]]%in%c("mm")){paleta=brewer.pal(11,"BrBG"); range=c(-0.25,0.25)}
      if(di[[names(di)]][["Units"]]%in%c("degrees_C","days","%")){paleta=rev(brewer.pal(11,"RdBu")); range=c(-0.4,0.4)}
      
      # png(filename = paste(OUTPUTS,"Trend_1950_2014_",ssn[s],"_",indices[i],"_",Model[m],".png",sep=""),res=600,width=3000,height=2000)
      # if(dim(subset(datita,Significance<0.05))[1]!=0){
      #   print(ggplot(data=datita,aes(x=Longitude,y=Latitude,fill=Trend)) + geom_contour_fill(bins=20)  +  
      #           scale_fill_gradientn(name=paste(di[[names(di)]][["Units"]],"decade",sep="/"),colours=paleta,limits=range,na.value = "white")+
      #           coord_quickmap(xlim=c(min(datita$Longitude),max(datita$Longitude)),
      #                          ylim=c(min(datita$Latitude),max(datita$Latitude)),expand=F)+ borders() + 
      #           theme_bw() + ggtitle(paste(ssn[s],indices[i],Model[m],sep=" "))+geom_text(data=subset(datita,Significance<=0.05),aes(label="*",fontface="bold")))
      #   dev.off()}else{
      #     print(ggplot(data=datita,aes(x=Longitude,y=Latitude,fill=Trend)) + geom_contour_fill(bins=20)  +  
      #             scale_fill_gradientn(name=paste(di[[names(di)]][["Units"]],"decade",sep="/"),colours=paleta,limits=range,na.value = "white")+
      #             coord_quickmap(xlim=c(min(datita$Longitude),max(datita$Longitude)),
      #                            ylim=c(min(datita$Latitude),max(datita$Latitude)),expand=F)+ borders() + 
      #             theme_bw() + ggtitle(paste(ssn[s],indices[i],Model[m],sep=" ")))#+geom_text(data=subset(datita,Significance<=0.05),aes(label="*",fontface="bold")))
      #     
      #   }
    }
  }
}
save(trend.indices,file="trend.indices.mon.CMIP6.map.historical.Rdata")
save(series.indices,file="series.indices.mon.CMIP6.map.historical.Rdata")


######################################################
#--------------------Ensemble map----------------###
invariable.data <- data.frame(Longitude=rep(invariable$xyCoords$x,each=length(invariable$xyCoords$y)),
                              Latitude=rep(invariable$xyCoords$y,length(invariable$xyCoords$x)),
                              Invarible=as.numeric(redim(invariable,drop=T)$Data))#Annual

agreement <- function(x){
  sig <- sign(x)
  pos <- length(which(sig>0))/length(sig)*100
  neg <- length(which(sig<0))/length(sig)*100
  value <- max(pos,neg)
  if(value<50){ag <- 0}
  if(value>=50 & value<80){ag <- 1}
  if(value>=80){ag <- 2}
  return(ag)
}

#Annual
load("trend.indices.yr.CMIP6.map.historical.Rdata")

indices <- c("wsdi","csdi","tr",
             "r20mm","prcptot","cdd","cwd")

test <- list()
for(i in 1:length(indices)){
  test[[i]] <- list()
  for(m in 1:length(trend.indices[[i]])){
    test[[i]][[m]] <- do.call("rbind",trend.indices[[i]][[m]])
  }
}

test2 <- list()
for(i in 1:length(indices)){
  test2[[i]] <- do.call("rbind",test[[i]])
}

test3 <- do.call("rbind",test2)
test3$Model[which(test3$Model=="")] <- "ERA5"

Model <- unique(test3$Model)
Model <- Model[order(Model)]

test3$Model <- factor(test3$Model,levels=rev(c("ERA5",Model[-which(Model=="ERA5")])))
test3$Index <- toupper(test3$Index)
test3$Index <- factor(test3$Index,levels=toupper(c("cdd","cwd","prcptot","r20mm","csdi","wsdi","tr")))

ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)

ensemble2 <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "agreement")
ensemble$Agreement <- ensemble2$Tau

invariable.rep <- do.call("rbind", replicate(length(indices), invariable.data, simplify = FALSE))
ensemble$Agreement[which(is.na(invariable.rep$Invarible))] <- NaN

ensemble$Tau[which(ensemble$Index=="CDD")] <- ensemble$Tau[which(ensemble$Index=="CDD")]*-1

ensemble$Tau[which(ensemble$Tau>0.4)] <- 0.39
ensemble$Tau[which(ensemble$Tau< -0.4)] <- -0.39

p <- ggplot(data=subset(ensemble,Index%in%toupper(c("cdd","cwd","r20mm","prcptot"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=3)+
  scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
  coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                 ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
  theme_bw() + ggtitle("CMIP6 ensemble: annual indices")+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("cdd","cwd","r20mm","prcptot"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("cdd","cwd","r20mm","prcptot")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))

g <- ggplot(data=subset(ensemble,Index%in%toupper(c("csdi","wsdi","tr"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
  scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
  coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                 ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
  theme_bw() + ggtitle("CMIP6 ensemble: annual indices")+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("csdi","wsdi","tr"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("csdi","wsdi","tr")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))

pcat <- ggplot(data=subset(ensemble,Index%in%toupper(c("cdd","cwd","r20mm","prcptot"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=3)+
  scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
  coord_quickmap(xlim=c(0,4),
                 ylim=c(40,43),expand=F)+ borders() + 
  theme_bw() + ggtitle("CMIP6 ensemble: annual indices")+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("cdd","cwd","r20mm","prcptot"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("cdd","cwd","r20mm","prcptot")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))

gcat <- ggplot(data=subset(ensemble,Index%in%toupper(c("csdi","wsdi","tr"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
  scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
  coord_quickmap(xlim=c(0,4),
                 ylim=c(40,43),expand=F)+ borders() + 
  theme_bw() + ggtitle("CMIP6 ensemble: annual indices")+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("csdi","wsdi","tr"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("csdi","wsdi","tr")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))


###ERA5---#
ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
ensemble$Tau[which(ensemble$Index=="CDD")] <- ensemble$Tau[which(ensemble$Index=="CDD")]*-1

ensemble$Tau[which(ensemble$Tau>0.4)] <- 0.39
ensemble$Tau[which(ensemble$Tau< -0.4)] <- -0.39

h <- ggplot(data=subset(ensemble,Index%in%toupper(c("cdd","cwd","r20mm","prcptot"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20) + facet_wrap(~Index,nrow=3)+
  scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
  coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                 ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
  theme_bw() + ggtitle("ERA5")

i <- ggplot(data=subset(ensemble,Index%in%toupper(c("csdi","wsdi","tr"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
  scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
  coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                 ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
  theme_bw() + ggtitle("ERA5")

hcat <- ggplot(data=subset(ensemble,Index%in%toupper(c("cdd","cwd","r20mm","prcptot"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20) + facet_wrap(~Index,nrow=3)+
  scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
  coord_quickmap(xlim=c(0,4),
                 ylim=c(40,43),expand=F)+ borders() + 
  theme_bw() + ggtitle("ERA5")

icat <- ggplot(data=subset(ensemble,Index%in%toupper(c("csdi","wsdi","tr"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
  scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
  coord_quickmap(xlim=c(0,4),
                 ylim=c(40,43),expand=F)+ borders() + 
  theme_bw() + ggtitle("ERA5")

png(filename = paste(OUTPUTS,"Indices_CMIP6_Trends_1950-2014_Map_Annual_EuroMediterranean",".png",sep=""),res=300,width=4000,height=2000)
print(
  grid.arrange(p,h,g,i,nrow=2)
)
dev.off() 

png(filename = paste(OUTPUTS,"Indices_CMIP6_Trends_1950-2014_Map_Annual_Catalonia",".png",sep=""),res=300,width=4000,height=2000)
print(
  grid.arrange(pcat,hcat,gcat,icat,nrow=2)
)
dev.off() 

#---------------------------------------------------------#
#Seasonal
load("trend.indices.mon.CMIP6.map.historical.Rdata")

indices <- c("txx","txn","tnx","tnn","tx90p","tx10p","tn90p","tn10p",
             "rx5day","rx1day")
ssn <- c("Winter","Spring","Summer","Autumn")#,"Annual")

test <- list()
for(i in 1:length(indices)){
  test[[i]] <- list()
  for(m in 1:length(trend.indices[[i]])){
    test[[i]][[m]] <- do.call("rbind",trend.indices[[i]][[m]])
  }
}

test2 <- list()
for(i in 1:length(indices)){
  test2[[i]] <- do.call("rbind",test[[i]])
}

test3 <- do.call("rbind",test2)
test3$Index <- toupper(test3$Index)
test3$Model[which(test3$Model=="")] <- "ERA5"

Model <- unique(test3$Model)
Model <- Model[order(Model)]

test3$Model <- factor(test3$Model,levels=rev(c("ERA5",Model[-which(Model=="ERA5")])))
test3$Index <- factor(test3$Index,levels=toupper(c("tnn","tnx","txn","txx","tn10p","tn90p","tx10p","tx90p",
                                                   "rx1day","rx5day")))

test3$Season <- factor(test3$Season,levels=c("Summer","Autumn","Winter","Spring"))

tes.t <- test3

for(j in 1:length(ssn)){
  test3 <- subset(tes.t,Season==ssn[j])
  
  #precip
  ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
  
  ensemble2 <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "agreement")
  ensemble$Agreement <- ensemble2$Tau
  
  invariable.rep <- do.call("rbind", replicate(length(indices), invariable.data, simplify = FALSE))
  ensemble$Agreement[which(is.na(invariable.rep$Invarible))] <- NaN
  
  ensemble$Tau[which(ensemble$Tau>0.2)] <- 0.19
  ensemble$Tau[which(ensemble$Tau< -0.2)] <- -0.19
  
  p <- ggplot(data=subset(ensemble,Index%in%toupper(c("rx1day","rx5day"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CMIP6 ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("rx1day","rx5day"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("rx1day","rx5day")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  pcat <- ggplot(data=subset(ensemble,Index%in%toupper(c("rx1day","rx5day"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CMIP6 ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("rx1day","rx5day"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("rx1day","rx5day")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  #temp
  ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
  
  ensemble2 <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[-which(unique(test3$Model)=="ERA5")]),FUN = "agreement")
  ensemble$Agreement <- ensemble2$Tau
  
  invariable.rep <- do.call("rbind", replicate(length(indices), invariable.data, simplify = FALSE))
  ensemble$Agreement[which(is.na(invariable.rep$Invarible))] <- NaN
  
  ensemble$Tau[which(ensemble$Tau>0.4)] <- 0.39
  ensemble$Tau[which(ensemble$Tau< -0.4)] <- -0.39
  
  g1 <- ggplot(data=subset(ensemble,Index%in%toupper(c("tnn","tnx","txn","txx"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CMIP6 ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("tnn","tnx","txn","txx"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("tnn","tnx","txn","txx")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  g2 <- ggplot(data=subset(ensemble,Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CMIP6 ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  
  g1cat <- ggplot(data=subset(ensemble,Index%in%toupper(c("tnn","tnx","txn","txx"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CMIP6 ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("tnn","tnx","txn","txx"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("tnn","tnx","txn","txx")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  g2cat <- ggplot(data=subset(ensemble,Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("CMIP6 ensemble",ssn[j],sep=" "))+geom_text(data=subset(ensemble,Agreement>0 & Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),size=subset(ensemble,Agreement>0 & Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p")))$Agreement,aes(label="*",fontface="bold"))#+geom_text(aes(label="*",fontface="bold",lty))
  
  ###ERA5---#
  
  #precip
  ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
  
  ensemble$Tau[which(ensemble$Tau>0.2)] <- 0.19
  ensemble$Tau[which(ensemble$Tau< -0.2)] <- -0.19
  
  h <- ggplot(data=subset(ensemble,Index%in%toupper(c("rx1day","rx5day"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20) + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  hcat <- ggplot(data=subset(ensemble,Index%in%toupper(c("rx1day","rx5day"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20) + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  #---Catalonia---#
  hcat <- ggplot(data=subset(ensemble,Index%in%toupper(c("rx1day","rx5day"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20) + facet_wrap(~Index,nrow=1)+
    scale_fill_gradientn(name="Tau",colours=brewer.pal(11,"BrBG"),limits=c(-0.2,0.2),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  #temp
  ensemble <- aggregate(Tau~Latitude+Longitude+Index,subset(test3,Model%in%unique(test3$Model)[which(unique(test3$Model)=="ERA5")]),FUN = "mean",na.rm=T)
  
  ensemble$Tau[which(ensemble$Tau>0.4)] <- 0.39
  ensemble$Tau[which(ensemble$Tau< -0.4)] <- -0.39
  
  i <- ggplot(data=subset(ensemble,Index%in%toupper(c("tnn","tnx","txn","txx"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  j1 <- ggplot(data=subset(ensemble,Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(min(ensemble$Longitude),max(ensemble$Longitude)),
                   ylim=c(min(ensemble$Latitude),max(ensemble$Latitude)),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  icat <- ggplot(data=subset(ensemble,Index%in%toupper(c("tnn","tnx","txn","txx"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  j1cat <- ggplot(data=subset(ensemble,Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  icat <- ggplot(data=subset(ensemble,Index%in%toupper(c("tnn","tnx","txn","txx"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  j1cat <- ggplot(data=subset(ensemble,Index%in%toupper(c("tn10p","tn90p","tx10p","tx90p"))),aes(x=Longitude,y=Latitude,z=Tau)) + geom_contour_fill(bins=20)  + facet_wrap(~Index,nrow=2)+
    scale_fill_gradientn(name="Tau",colours=rev(brewer.pal(11,"RdBu")),limits=c(-0.4,0.4),na.value = "white")+
    coord_quickmap(xlim=c(0,4),
                   ylim=c(40,43),expand=F)+ borders() + 
    theme_bw() + ggtitle(paste("ERA5",ssn[j],sep=" "))
  
  png(filename = paste(OUTPUTS,"Indices_TX_TN_CMIP6_Trends_1950-2014_Map_",ssn[j],"_EuroMediterranean.png",sep=""),res=300,width=4000,height=2500)
  print(
    grid.arrange(#p,h,
      g1,i,
      g2,j1,nrow=2)
  )
  dev.off() 
  
  png(filename = paste(OUTPUTS,"Indices_PR_CMIP6_Trends_1950-2014_Map_",ssn[j],"_EuroMediterranean.png",sep=""),res=300,width=4000,height=2500)
  print(
    grid.arrange(p,h,p,h,
                 nrow=2)
  )
  dev.off() 
  
  png(filename = paste(OUTPUTS,"Indices_TX_TN_CMIP6_Trends_1950-2014_Map_",ssn[j],"_Catalonia.png",sep=""),res=300,width=4000,height=2500)
  print(
    grid.arrange(#p,h,
      g1cat,icat,
      g2cat,j1cat,nrow=2)
  )
  dev.off() 
  
  png(filename = paste(OUTPUTS,"Indices_PR_CMIP6_Trends_1950-2014_Map_",ssn[j],"_Catalonia.png",sep=""),res=300,width=4000,height=2500)
  print(
    grid.arrange(pcat,hcat,pcat,hcat,
                 nrow=2)
  )
  dev.off() 
}
